@@ -28,15 +28,14 @@
  * MT safe
  */
 
+#include "config.h"
+
 #define DEBUG_MSG(x)	/* */
 #ifdef G_ENABLE_DEBUG
 /* #define DEBUG_MSG(args)	g_message args ; */
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "galias.h"
 #include "glib.h"
 
 #include <time.h>
@@ -45,7 +44,7 @@
 #include <locale.h>
 
 GDate*
-g_date_new ()
+g_date_new (void)
 {
   GDate *d = g_new0 (GDate, 1); /* happily, 0 is the invalid flag for everything. */
   
@@ -396,6 +395,42 @@ g_date_get_sunday_week_of_year (const GDate *d)
   return ((day + wd)/7U + (wd == 0 ? 1 : 0));
 }
 
+/**
+ * g_date_get_iso8601_week_of_year:
+ * @date: a valid #GDate
+ *
+ * Returns the week of the year, where weeks are interpreted according
+ * to ISO 8601. 
+ * 
+ * Returns: ISO 8601 week number of the year.
+ *
+ * Since: 2.6
+ **/
+guint
+g_date_get_iso8601_week_of_year (const GDate *d)
+{
+  guint j, d4, L, d1, w;
+
+  g_return_val_if_fail (d != NULL, 0);
+  g_return_val_if_fail (g_date_valid (d), 0);
+  
+  if (!d->julian)
+    g_date_update_julian (d);
+  g_return_val_if_fail (d->julian, 0);
+
+  /* Formula taken from the Calendar FAQ; the formula was for the
+   * Julian Period which starts on 1 January 4713 BC, so we add
+   * 1,721,425 to the number of days before doing the formula. 
+   */
+  j  = d->julian + 1721425;
+  d4 = (j + 31741 - (j % 7)) % 146097 % 36524 % 1461;
+  L  = d4 / 1460;
+  d1 = ((d4 - L) % 365) + L;
+  w  = d1 / 7 + 1;
+
+  return w;
+}
+
 gint
 g_date_days_between (const GDate *d1,
 		     const GDate *d2)
@@ -456,6 +491,10 @@ static GDateYear twodigit_start_year = 1930;
  */
 static gboolean using_twodigit_years = FALSE;
 
+/* Adjustment of locale era to AD, non-zero means using locale era
+ */
+static gint locale_era_adjust = 0;
+
 struct _GDateParseTokens {
   gint num_ints;
   gint n[3];
@@ -528,7 +567,7 @@ g_date_fill_parse_tokens (const gchar *str, GDateParseTokens *pt)
               if (found != NULL)
                 {
                   pt->month = i;
-		  return;
+		  break;
                 }
             }
 	  
@@ -539,12 +578,14 @@ g_date_fill_parse_tokens (const gchar *str, GDateParseTokens *pt)
               if (found != NULL)
                 {
                   pt->month = i;
-                  return;
+		  break;
                 }
             }
 
           ++i;
-        }      
+        }
+
+      g_free (normalized);
     }
 }
 
@@ -626,7 +667,9 @@ g_date_prepare_to_parse (const gchar *str, GDateParseTokens *pt)
               dmy_order[i] = G_DATE_YEAR;
               break;
             default:
-              /* leave it unchanged */
+              /* assume locale era */
+              locale_era_adjust = 1976 - testpt.n[i];
+              dmy_order[i] = G_DATE_YEAR;
               break;
             }
           ++i;
@@ -737,7 +780,11 @@ g_date_set_parse (GDate       *d,
 	    {
 	      y  = pt.n[i];
 	      
-	      if (using_twodigit_years && y < 100)
+	      if (locale_era_adjust != 0)
+	        {
+		  y += locale_era_adjust;
+	        }
+	      else if (using_twodigit_years && y < 100)
 		{
 		  guint two     =  twodigit_start_year % 100;
 		  guint century = (twodigit_start_year / 100) * 100;
@@ -837,8 +884,22 @@ g_date_set_time (GDate *d,
 #else
   {
     struct tm *ptm = localtime (&t);
-    g_assert (ptm);
-    memcpy ((void *) &tm, (void *) ptm, sizeof(struct tm));
+
+    if (ptm == NULL)
+      {
+	/* Happens at least in Microsoft's C library if you pass a
+	 * negative time_t. Use 2000-01-01 as default date.
+	 */
+#ifndef G_DISABLE_CHECKS
+	g_return_if_fail_warning (G_LOG_DOMAIN, "g_date_set_time", "ptm != NULL");
+#endif
+
+	tm.tm_mon = 0;
+	tm.tm_mday = 1;
+	tm.tm_year = 100;
+      }
+    else
+      memcpy ((void *) &tm, (void *) ptm, sizeof(struct tm));
   }
 #endif
   
