@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include "galias.h"
 #include "glib.h"
 
 #include <sys/stat.h>
@@ -37,21 +38,8 @@
 #include <stdlib.h>
 
 #ifdef G_OS_WIN32
+#include <windows.h>
 #include <io.h>
-#ifndef F_OK
-#define	F_OK 0
-#define	W_OK 2
-#define	R_OK 4
-#endif /* !F_OK */
-
-#ifndef S_ISREG
-#define S_ISREG(mode) ((mode)&_S_IFREG)
-#endif
-
-#ifndef S_ISDIR
-#define S_ISDIR(mode) ((mode)&_S_IFDIR)
-#endif
-
 #endif /* G_OS_WIN32 */
 
 #ifndef S_ISLNK
@@ -62,17 +50,12 @@
 #define O_BINARY 0
 #endif
 
+#include "gstdio.h"
 #include "glibintl.h"
 
-#ifdef G_OS_WIN32
-#define G_IS_DIR_SEPARATOR(c) (c == G_DIR_SEPARATOR || c == '/')
-#else
-#define G_IS_DIR_SEPARATOR(c) (c == G_DIR_SEPARATOR)
-#endif
-
 /**
  * g_file_test:
- * @filename: a filename to test
+ * @filename: a filename to test in the GLib file name encoding
  * @test: bitfield of #GFileTest flags
  * 
  * Returns %TRUE if any of the tests in the bitfield @test are
@@ -90,7 +73,7 @@
  * %TRUE for %G_FILE_TEST_IS_SYMLINK and %FALSE for all other flags.
  *
  * You should never use g_file_test() to test whether it is safe
- * to perform an operaton, because there is always the possibility
+ * to perform an operation, because there is always the possibility
  * of the condition changing before you actually perform the operation.
  * For example, you might think you could use %G_FILE_TEST_IS_SYMLINK
  * to know whether it is is safe to write to a file without being
@@ -99,7 +82,7 @@
  * <informalexample><programlisting>
  * /&ast; DON'T DO THIS &ast;/
  *  if (!g_file_test (filename, G_FILE_TEST_IS_SYMLINK)) {
- *    fd = open (filename, O_WRONLY);
+ *    fd = g_open (filename, O_WRONLY);
  *    /&ast; write to fd &ast;/
  *  }
  * </programlisting></informalexample>
@@ -108,19 +91,121 @@
  * %G_FILE_TEST_IS_EXECUTABLE are implemented using the access()
  * system call. This usually doesn't matter, but if your program
  * is setuid or setgid it means that these tests will give you
- * the answer for the real user ID and group ID , rather than the
+ * the answer for the real user ID and group ID, rather than the
  * effective user ID and group ID.
  *
+ * On Windows, there are no symlinks, so testing for
+ * %G_FILE_TEST_IS_SYMLINK will always return %FALSE. Testing for
+ * %G_FILE_TEST_IS_EXECUTABLE will just check that the file exists and
+ * its name indicates that it is executable, checking for well-known
+ * extensions and those listed in the %PATHEXT environment variable.
+ *
  * Return value: whether a test was %TRUE
  **/
 gboolean
 g_file_test (const gchar *filename,
              GFileTest    test)
 {
+#ifdef G_OS_WIN32
+/* stuff missing in std vc6 api */
+#  ifndef INVALID_FILE_ATTRIBUTES
+#    define INVALID_FILE_ATTRIBUTES -1
+#  endif
+#  ifndef FILE_ATTRIBUTE_DEVICE
+#    define FILE_ATTRIBUTE_DEVICE 64
+#  endif
+  int attributes;
+
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+
+      if (wfilename == NULL)
+	return FALSE;
+
+      attributes = GetFileAttributesW (wfilename);
+
+      g_free (wfilename);
+    }
+  else
+    {
+      gchar *cpfilename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+
+      if (cpfilename == NULL)
+	return FALSE;
+      
+      attributes = GetFileAttributesA (cpfilename);
+      
+      g_free (cpfilename);
+    }
+
+  if (attributes == INVALID_FILE_ATTRIBUTES)
+    return FALSE;
+
+  if (test & G_FILE_TEST_EXISTS)
+    return TRUE;
+      
+  if (test & G_FILE_TEST_IS_REGULAR)
+    return (attributes & (FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_DEVICE)) == 0;
+
+  if (test & G_FILE_TEST_IS_DIR)
+    return (attributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
+
+  if (test & G_FILE_TEST_IS_EXECUTABLE)
+    {
+      const gchar *lastdot = strrchr (filename, '.');
+      const gchar *pathext = NULL, *p;
+      int extlen;
+
+      if (lastdot == NULL)
+	return FALSE;
+
+      if (stricmp (lastdot, ".exe") == 0 ||
+	  stricmp (lastdot, ".cmd") == 0 ||
+	  stricmp (lastdot, ".bat") == 0 ||
+	  stricmp (lastdot, ".com") == 0)
+	return TRUE;
+
+      /* Check if it is one of the types listed in %PATHEXT% */
+
+      pathext = g_getenv ("PATHEXT");
+      if (pathext == NULL)
+	return FALSE;
+
+      pathext = g_utf8_casefold (pathext, -1);
+
+      lastdot = g_utf8_casefold (lastdot, -1);
+      extlen = strlen (lastdot);
+
+      p = pathext;
+      while (TRUE)
+	{
+	  const gchar *q = strchr (p, ';');
+	  if (q == NULL)
+	    q = p + strlen (p);
+	  if (extlen == q - p &&
+	      memcmp (lastdot, p, extlen) == 0)
+	    {
+	      g_free ((gchar *) pathext);
+	      g_free ((gchar *) lastdot);
+	      return TRUE;
+	    }
+	  if (*q)
+	    p = q + 1;
+	  else
+	    break;
+	}
+
+      g_free ((gchar *) pathext);
+      g_free ((gchar *) lastdot);
+      return FALSE;
+    }
+
+  return FALSE;
+#else
   if ((test & G_FILE_TEST_EXISTS) && (access (filename, F_OK) == 0))
     return TRUE;
   
-#ifndef G_OS_WIN32
   if ((test & G_FILE_TEST_IS_EXECUTABLE) && (access (filename, X_OK) == 0))
     {
       if (getuid () != 0)
@@ -133,18 +218,13 @@ g_file_test (const gchar *filename,
     }
   else
     test &= ~G_FILE_TEST_IS_EXECUTABLE;
-#endif	
 
   if (test & G_FILE_TEST_IS_SYMLINK)
     {
-#ifdef G_OS_WIN32
-      /* no sym links on win32, no lstat in msvcrt */
-#else
       struct stat s;
 
       if ((lstat (filename, &s) == 0) && S_ISLNK (s.st_mode))
         return TRUE;
-#endif
     }
   
   if (test & (G_FILE_TEST_IS_REGULAR |
@@ -161,26 +241,45 @@ g_file_test (const gchar *filename,
 	  if ((test & G_FILE_TEST_IS_DIR) && S_ISDIR (s.st_mode))
 	    return TRUE;
 
-#ifndef G_OS_WIN32
 	  /* The extra test for root when access (file, X_OK) succeeds.
-	   * Probably only makes sense on Unix.
 	   */
 	  if ((test & G_FILE_TEST_IS_EXECUTABLE) &&
 	      ((s.st_mode & S_IXOTH) ||
 	       (s.st_mode & S_IXUSR) ||
 	       (s.st_mode & S_IXGRP)))
 	    return TRUE;
-#else
-	  if ((test & G_FILE_TEST_IS_EXECUTABLE) &&
-	      (s.st_mode & _S_IEXEC))
-	    return TRUE;
-#endif
 	}
     }
 
   return FALSE;
+#endif
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_file_test
+
+/* Binary compatibility version. Not for newly compiled code. */
+
+gboolean
+g_file_test (const gchar *filename,
+             GFileTest    test)
+{
+  gchar *utf8_filename = g_locale_to_utf8 (filename, -1, NULL, NULL, NULL);
+  gboolean retval;
+
+  if (utf8_filename == NULL)
+    return FALSE;
+
+  retval = g_file_test_utf8 (utf8_filename, test);
+
+  g_free (utf8_filename);
+
+  return retval;
+}
+
+#endif
+
 GQuark
 g_file_error_quark (void)
 {
@@ -348,7 +447,13 @@ g_file_error_from_errno (gint err_no)
       return G_FILE_ERROR_PERM;
       break;
 #endif
-      
+
+#ifdef ENOSYS
+    case ENOSYS:
+      return G_FILE_ERROR_NOSYS;
+      break;
+#endif
+
     default:
       return G_FILE_ERROR_FAILED;
       break;
@@ -356,7 +461,7 @@ g_file_error_from_errno (gint err_no)
 }
 
 static gboolean
-get_contents_stdio (const gchar *filename,
+get_contents_stdio (const gchar *display_filename,
                     FILE        *f,
                     gchar      **contents,
                     gsize       *length, 
@@ -378,7 +483,10 @@ get_contents_stdio (const gchar *filename,
   
   while (!feof (f))
     {
+      int save_errno;
+
       bytes = fread (buf, 1, 2048, f);
+      save_errno = errno;
 
       while ((total_bytes + bytes + 1) > total_allocated)
         {
@@ -387,15 +495,12 @@ get_contents_stdio (const gchar *filename,
 
           if (str == NULL)
             {
-	      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-							 NULL, NULL, NULL);
               g_set_error (error,
                            G_FILE_ERROR,
                            G_FILE_ERROR_NOMEM,
                            _("Could not allocate %lu bytes to read file \"%s\""),
                            (gulong) total_allocated, 
-			   utf8_filename ? utf8_filename : "???");
-	      g_free (utf8_filename);
+			   display_filename);
 
               goto error;
             }
@@ -403,15 +508,12 @@ get_contents_stdio (const gchar *filename,
       
       if (ferror (f))
         {
-	  gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						     NULL, NULL, NULL);
           g_set_error (error,
                        G_FILE_ERROR,
-                       g_file_error_from_errno (errno),
+                       g_file_error_from_errno (save_errno),
                        _("Error reading file '%s': %s"),
-                       utf8_filename ? utf8_filename : "???", 
-		       g_strerror (errno));
-	  g_free (utf8_filename);
+                       display_filename,
+		       g_strerror (save_errno));
 
           goto error;
         }
@@ -442,7 +544,7 @@ get_contents_stdio (const gchar *filename,
 #ifndef G_OS_WIN32
 
 static gboolean
-get_contents_regfile (const gchar *filename,
+get_contents_regfile (const gchar *display_filename,
                       struct stat *stat_buf,
                       gint         fd,
                       gchar      **contents,
@@ -461,15 +563,12 @@ get_contents_regfile (const gchar *filename,
 
   if (buf == NULL)
     {
-      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						 NULL, NULL, NULL);
       g_set_error (error,
                    G_FILE_ERROR,
                    G_FILE_ERROR_NOMEM,
                    _("Could not allocate %lu bytes to read file \"%s\""),
                    (gulong) alloc_size, 
-		   utf8_filename ? utf8_filename : "???");
-      g_free (utf8_filename);
+		   display_filename);
 
       goto error;
     }
@@ -485,16 +584,15 @@ get_contents_regfile (const gchar *filename,
         {
           if (errno != EINTR) 
             {
-	      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-							 NULL, NULL, NULL);
+	      int save_errno = errno;
+
               g_free (buf);
               g_set_error (error,
                            G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
+                           g_file_error_from_errno (save_errno),
                            _("Failed to read from file '%s': %s"),
-                           utf8_filename ? utf8_filename : "???", 
-			   g_strerror (errno));
-	      g_free (utf8_filename);
+                           display_filename, 
+			   g_strerror (save_errno));
 
 	      goto error;
             }
@@ -531,21 +629,22 @@ get_contents_posix (const gchar *filename,
 {
   struct stat stat_buf;
   gint fd;
-  
+  gchar *display_filename = g_filename_display_name (filename);
+
   /* O_BINARY useful on Cygwin */
   fd = open (filename, O_RDONLY|O_BINARY);
 
   if (fd < 0)
     {
-      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						 NULL, NULL, NULL);
+      int save_errno = errno;
+
       g_set_error (error,
                    G_FILE_ERROR,
-                   g_file_error_from_errno (errno),
+                   g_file_error_from_errno (save_errno),
                    _("Failed to open file '%s': %s"),
-                   utf8_filename ? utf8_filename : "???", 
-		   g_strerror (errno));
-      g_free (utf8_filename);
+                   display_filename, 
+		   g_strerror (save_errno));
+      g_free (display_filename);
 
       return FALSE;
     }
@@ -553,52 +652,58 @@ get_contents_posix (const gchar *filename,
   /* I don't think this will ever fail, aside from ENOMEM, but. */
   if (fstat (fd, &stat_buf) < 0)
     {
-      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						 NULL, NULL, NULL);
+      int save_errno = errno;
+
       close (fd);
       g_set_error (error,
                    G_FILE_ERROR,
-                   g_file_error_from_errno (errno),
+                   g_file_error_from_errno (save_errno),
                    _("Failed to get attributes of file '%s': fstat() failed: %s"),
-                   utf8_filename ? utf8_filename : "???", 
-		   g_strerror (errno));
-      g_free (utf8_filename);
+                   display_filename, 
+		   g_strerror (save_errno));
+      g_free (display_filename);
 
       return FALSE;
     }
 
   if (stat_buf.st_size > 0 && S_ISREG (stat_buf.st_mode))
     {
-      return get_contents_regfile (filename,
-                                   &stat_buf,
-                                   fd,
-                                   contents,
-                                   length,
-                                   error);
+      gboolean retval = get_contents_regfile (display_filename,
+					      &stat_buf,
+					      fd,
+					      contents,
+					      length,
+					      error);
+      g_free (display_filename);
+
+      return retval;
     }
   else
     {
       FILE *f;
+      gboolean retval;
 
       f = fdopen (fd, "r");
       
       if (f == NULL)
         {
-	  gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						     NULL, NULL, NULL);
+	  int save_errno = errno;
 
           g_set_error (error,
                        G_FILE_ERROR,
-                       g_file_error_from_errno (errno),
+                       g_file_error_from_errno (save_errno),
                        _("Failed to open file '%s': fdopen() failed: %s"),
-                       utf8_filename ? utf8_filename : "???", 
-		       g_strerror (errno));
-          g_free (utf8_filename);
+                       display_filename, 
+		       g_strerror (save_errno));
+          g_free (display_filename);
 
           return FALSE;
         }
   
-      return get_contents_stdio (filename, f, contents, length, error);
+      retval = get_contents_stdio (display_filename, f, contents, length, error);
+      g_free (display_filename);
+
+      return retval;
     }
 }
 
@@ -606,52 +711,60 @@ get_contents_posix (const gchar *filename,
 
 static gboolean
 get_contents_win32 (const gchar *filename,
-                    gchar      **contents,
-                    gsize       *length,
-                    GError     **error)
+		    gchar      **contents,
+		    gsize       *length,
+		    GError     **error)
 {
   FILE *f;
-
-  /* I guess you want binary mode; maybe you want text sometimes? */
-  f = fopen (filename, "rb");
+  gboolean retval;
+  wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+  gchar *display_filename = g_filename_display_name (filename);
+  int save_errno;
+  
+  f = _wfopen (wfilename, L"rb");
+  save_errno = errno;
+  g_free (wfilename);
 
   if (f == NULL)
     {
-      gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						 NULL, NULL, NULL);
-      
       g_set_error (error,
                    G_FILE_ERROR,
-                   g_file_error_from_errno (errno),
+                   g_file_error_from_errno (save_errno),
                    _("Failed to open file '%s': %s"),
-                   utf8_filename ? utf8_filename : "???", 
-		   g_strerror (errno));
-      g_free (utf8_filename);
+                   display_filename,
+		   g_strerror (save_errno));
+      g_free (display_filename);
 
       return FALSE;
     }
   
-  return get_contents_stdio (filename, f, contents, length, error);
+  retval = get_contents_stdio (display_filename, f, contents, length, error);
+  g_free (display_filename);
+
+  return retval;
 }
 
 #endif
 
 /**
  * g_file_get_contents:
- * @filename: a file to read contents from
+ * @filename: name of a file to read contents from, in the GLib file name encoding
  * @contents: location to store an allocated string
- * @length: location to store length in bytes of the contents
- * @error: return location for a #GError
+ * @length: location to store length in bytes of the contents, or %NULL
+ * @error: return location for a #GError, or %NULL
  * 
  * Reads an entire file into allocated memory, with good error
- * checking. If @error is set, %FALSE is returned, and @contents is set
- * to %NULL. If %TRUE is returned, @error will not be set, and @contents
- * will be set to the file contents.  The string stored in @contents
- * will be nul-terminated, so for text files you can pass %NULL for the
- * @length argument.  The error domain is #G_FILE_ERROR. Possible
- * error codes are those in the #GFileError enumeration.
+ * checking. 
  *
- * Return value: %TRUE on success, %FALSE if error is set
+ * If the call was successful, it returns %TRUE and sets @contents to the file 
+ * contents and @length to the length of the file contents in bytes. The string 
+ * stored in @contents will be nul-terminated, so for text files you can pass 
+ * %NULL for the @length argument. If the call was not successful, it returns 
+ * %FALSE and sets @error. The error domain is #G_FILE_ERROR. Possible error  
+ * codes are those in the #GFileError enumeration. In the error case, 
+ * @contents is set to %NULL and @length is set to zero.
+ *
+ * Return value: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
 g_file_get_contents (const gchar *filename,
@@ -673,6 +786,33 @@ g_file_get_contents (const gchar *filename,
 #endif
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_file_get_contents
+
+/* Binary compatibility version. Not for newly compiled code. */
+
+gboolean
+g_file_get_contents (const gchar *filename,
+                     gchar      **contents,
+                     gsize       *length,
+                     GError     **error)
+{
+  gchar *utf8_filename = g_locale_to_utf8 (filename, -1, NULL, NULL, error);
+  gboolean retval;
+
+  if (utf8_filename == NULL)
+    return FALSE;
+
+  retval = g_file_get_contents (utf8_filename, contents, length, error);
+
+  g_free (utf8_filename);
+
+  return retval;
+}
+
+#endif
+
 /*
  * mkstemp() implementation is from the GNU C library.
  * Copyright (C) 1991,92,93,94,95,96,97,98,99 Free Software Foundation, Inc.
@@ -689,6 +829,8 @@ g_file_get_contents (const gchar *filename,
  * The parameter is a string that should match the rules for
  * mkstemp(), i.e. end in "XXXXXX". The X string will 
  * be modified to form the name of a file that didn't exist.
+ * The string should be in the GLib file name encoding. Most importantly, 
+ * on Windows it should be in UTF-8.
  *
  * Return value: A file handle (as from open()) to the file
  * opened for reading and writing. The file is opened in binary mode
@@ -713,7 +855,10 @@ g_mkstemp (gchar *tmpl)
 
   len = strlen (tmpl);
   if (len < 6 || strcmp (&tmpl[len - 6], "XXXXXX"))
-    return -1;
+    {
+      errno = EINVAL;
+      return -1;
+    }
 
   /* This is where the Xs start.  */
   XXXXXX = &tmpl[len - 6];
@@ -739,6 +884,77 @@ g_mkstemp (gchar *tmpl)
       v /= NLETTERS;
       XXXXXX[5] = letters[v % NLETTERS];
 
+      /* tmpl is in UTF-8 on Windows, thus use g_open() */
+      fd = g_open (tmpl, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
+
+      if (fd >= 0)
+	return fd;
+      else if (errno != EEXIST)
+	/* Any other error will apply also to other names we might
+	 *  try, and there are 2^32 or so of them, so give up now.
+	 */
+	return -1;
+    }
+
+  /* We got out of the loop because we ran out of combinations to try.  */
+  errno = EEXIST;
+  return -1;
+#endif
+}
+
+#ifdef G_OS_WIN32
+
+#undef g_mkstemp
+
+/* Binary compatibility version. Not for newly compiled code. */
+
+gint
+g_mkstemp (gchar *tmpl)
+{
+  int len;
+  char *XXXXXX;
+  int count, fd;
+  static const char letters[] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+  static const int NLETTERS = sizeof (letters) - 1;
+  glong value;
+  GTimeVal tv;
+  static int counter = 0;
+
+  len = strlen (tmpl);
+  if (len < 6 || strcmp (&tmpl[len - 6], "XXXXXX"))
+    {
+      errno = EINVAL;
+      return -1;
+    }
+
+  /* This is where the Xs start.  */
+  XXXXXX = &tmpl[len - 6];
+
+  /* Get some more or less random data.  */
+  g_get_current_time (&tv);
+  value = (tv.tv_usec ^ tv.tv_sec) + counter++;
+
+  for (count = 0; count < 100; value += 7777, ++count)
+    {
+      glong v = value;
+
+      /* Fill in the random bits.  */
+      XXXXXX[0] = letters[v % NLETTERS];
+      v /= NLETTERS;
+      XXXXXX[1] = letters[v % NLETTERS];
+      v /= NLETTERS;
+      XXXXXX[2] = letters[v % NLETTERS];
+      v /= NLETTERS;
+      XXXXXX[3] = letters[v % NLETTERS];
+      v /= NLETTERS;
+      XXXXXX[4] = letters[v % NLETTERS];
+      v /= NLETTERS;
+      XXXXXX[5] = letters[v % NLETTERS];
+
+      /* This is the backward compatibility system codepage version,
+       * thus use normal open().
+       */
       fd = open (tmpl, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
 
       if (fd >= 0)
@@ -751,10 +967,12 @@ g_mkstemp (gchar *tmpl)
     }
 
   /* We got out of the loop because we ran out of combinations to try.  */
+  errno = EEXIST;
   return -1;
-#endif
 }
 
+#endif
+
 /**
  * g_file_open_tmp:
  * @tmpl: Template for file name, as in g_mkstemp(), basename only
@@ -764,17 +982,18 @@ g_mkstemp (gchar *tmpl)
  * Opens a file for writing in the preferred directory for temporary
  * files (as returned by g_get_tmp_dir()). 
  *
- * @tmpl should be a string ending with six 'X' characters, as the
- * parameter to g_mkstemp() (or mkstemp()). 
- * However, unlike these functions, the template should only be a 
- * basename, no directory components are allowed. If template is %NULL, 
- * a default template is used.
+ * @tmpl should be a string in the GLib file name encoding ending with
+ * six 'X' characters, as the parameter to g_mkstemp() (or mkstemp()).
+ * However, unlike these functions, the template should only be a
+ * basename, no directory components are allowed. If template is
+ * %NULL, a default template is used.
  *
  * Note that in contrast to g_mkstemp() (and mkstemp()) 
  * @tmpl is not modified, and might thus be a read-only literal string.
  *
  * The actual name used is returned in @name_used if non-%NULL. This
  * string should be freed with g_free() when not needed any longer.
+ * The returned name is in the GLib file name encoding.
  *
  * Return value: A file handle (as from open()) to 
  * the file opened for reading and writing. The file is opened in binary 
@@ -802,6 +1021,7 @@ g_file_open_tmp (const gchar *tmpl,
 #endif
       )
     {
+      gchar *display_tmpl = g_filename_display_name (tmpl);
       char c[2];
       c[0] = *slash;
       c[1] = '\0';
@@ -810,7 +1030,8 @@ g_file_open_tmp (const gchar *tmpl,
 		   G_FILE_ERROR,
 		   G_FILE_ERROR_FAILED,
 		   _("Template '%s' invalid, should not contain a '%s'"),
-		   tmpl, c);
+		   display_tmpl, c);
+      g_free (display_tmpl);
 
       return -1;
     }
@@ -818,11 +1039,13 @@ g_file_open_tmp (const gchar *tmpl,
   if (strlen (tmpl) < 6 ||
       strcmp (tmpl + strlen (tmpl) - 6, "XXXXXX") != 0)
     {
+      gchar *display_tmpl = g_filename_display_name (tmpl);
       g_set_error (error,
 		   G_FILE_ERROR,
 		   G_FILE_ERROR_FAILED,
 		   _("Template '%s' doesn't end with XXXXXX"),
-		   tmpl);
+		   display_tmpl);
+      g_free (display_tmpl);
       return -1;
     }
 
@@ -839,11 +1062,15 @@ g_file_open_tmp (const gchar *tmpl,
 
   if (retval == -1)
     {
+      int save_errno = errno;
+      gchar *display_fulltemplate = g_filename_display_name (fulltemplate);
+
       g_set_error (error,
 		   G_FILE_ERROR,
-		   g_file_error_from_errno (errno),
+		   g_file_error_from_errno (save_errno),
 		   _("Failed to create file '%s': %s"),
-		   fulltemplate, g_strerror (errno));
+		   display_fulltemplate, g_strerror (save_errno));
+      g_free (display_fulltemplate);
       g_free (fulltemplate);
       return -1;
     }
@@ -856,6 +1083,39 @@ g_file_open_tmp (const gchar *tmpl,
   return retval;
 }
 
+#ifdef G_OS_WIN32
+
+#undef g_file_open_tmp
+
+/* Binary compatibility version. Not for newly compiled code. */
+
+gint
+g_file_open_tmp (const gchar *tmpl,
+		 gchar      **name_used,
+		 GError     **error)
+{
+  gchar *utf8_tmpl = g_locale_to_utf8 (tmpl, -1, NULL, NULL, error);
+  gchar *utf8_name_used;
+  gint retval;
+
+  if (utf8_tmpl == NULL)
+    return -1;
+
+  retval = g_file_open_tmp_utf8 (utf8_tmpl, &utf8_name_used, error);
+  
+  if (retval == -1)
+    return -1;
+
+  if (name_used)
+    *name_used = g_locale_from_utf8 (utf8_name_used, -1, NULL, NULL, NULL);
+
+  g_free (utf8_name_used);
+
+  return retval;
+}
+
+#endif
+
 static gchar *
 g_build_pathv (const gchar *separator,
 	       const gchar *first_element,
@@ -1154,9 +1414,9 @@ g_build_filename (const gchar *first_element,
  * @filename: the symbolic link
  * @error: return location for a #GError
  *
- * Reads the contents of the symbolic link @filename like the POSIX readlink() function.
- * The returned string is in the encoding used for filenames. Use g_filename_to_utf8() to 
- * convert it to UTF-8.
+ * Reads the contents of the symbolic link @filename like the POSIX
+ * readlink() function.  The returned string is in the encoding used
+ * for filenames. Use g_filename_to_utf8() to convert it to UTF-8.
  *
  * Returns: A newly allocated string with the contents of the symbolic link, 
  *          or %NULL if an error occurred.
@@ -1179,16 +1439,17 @@ g_file_read_link (const gchar *filename,
     {
       read_size = readlink (filename, buffer, size);
       if (read_size < 0) {
-	gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
-						   NULL, NULL, NULL);
+	int save_errno = errno;
+	gchar *display_filename = g_filename_display_name (filename);
+
 	g_free (buffer);
 	g_set_error (error,
 		     G_FILE_ERROR,
-		     g_file_error_from_errno (errno),
+		     g_file_error_from_errno (save_errno),
 		     _("Failed to read the symbolic link '%s': %s"),
-		     utf8_filename ? utf8_filename : "???", 
-		     g_strerror (errno));
-	g_free (utf8_filename);
+		     display_filename, 
+		     g_strerror (save_errno));
+	g_free (display_filename);
 	
 	return NULL;
       }
