@@ -22,6 +22,7 @@
  */
 
 #include "config.h"
+#include "galias.h"
 
 #include "gkeyfile.h"
 
@@ -60,8 +61,6 @@
 #include "gstrfuncs.h"
 #include "gutils.h"
 
-#include "galias.h"
-
 typedef struct _GKeyFileGroup GKeyFileGroup;
 
 struct _GKeyFile
@@ -152,9 +151,6 @@ static gint                  g_key_file_parse_value_as_integer (GKeyFile
 								GError                **error);
 static gchar                *g_key_file_parse_integer_as_value (GKeyFile               *key_file,
 								gint                    value);
-static gdouble               g_key_file_parse_value_as_double  (GKeyFile               *key_file,
-                                                                const gchar            *value,
-                                                                GError                **error);
 static gboolean              g_key_file_parse_value_as_boolean (GKeyFile               *key_file,
 								const gchar            *value,
 								GError                **error);
@@ -188,7 +184,12 @@ static void                  g_key_file_flush_parse_buffer     (GKeyFile
 GQuark
 g_key_file_error_quark (void)
 {
-  return g_quark_from_static_string ("g-key-file-error-quark");
+  static GQuark error_quark = 0;
+
+  if (error_quark == 0)
+    error_quark = g_quark_from_static_string ("g-key-file-error-quark");
+
+  return error_quark;
 }
 
 static void
@@ -341,8 +342,6 @@ find_file_in_data_dirs (const gchar   *file,
   if (output_file != NULL && fd > 0)
     *output_file = g_strdup (path);
 
-  g_free (path);
-
   return fd;
 }
 
@@ -353,7 +352,7 @@ g_key_file_load_from_fd (GKeyFile       *key_file,
 			 GError        **error)
 {
   GError *key_file_error = NULL;
-  gssize bytes_read;
+  gsize bytes_read;
   struct stat stat_buf;
   gchar read_buf[4096];
   
@@ -585,11 +584,8 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
 
   found_file = FALSE;
   data_dirs = all_data_dirs;
-  output_path = NULL;
   while (*data_dirs != NULL && !found_file)
     {
-      g_free (output_path);
-
       fd = find_file_in_data_dirs (file, &output_path, &data_dirs, 
                                    &key_file_error);
       
@@ -597,7 +593,7 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
         {
           if (key_file_error)
             g_propagate_error (error, key_file_error);
- 	  break;
+	  break;
         }
 
       found_file = g_key_file_load_from_fd (key_file, fd, flags,
@@ -607,17 +603,15 @@ g_key_file_load_from_data_dirs (GKeyFile       *key_file,
       if (key_file_error)
         {
 	  g_propagate_error (error, key_file_error);
+          g_free (output_path);
 	  break;
         }
+      
+      if (full_path)
+	*full_path = output_path;
     }
 
-  if (found_file && full_path)
-    *full_path = output_path;
-  else 
-    g_free (output_path);
-
   g_strfreev (all_data_dirs);
-
   return found_file;
 }
 
@@ -690,14 +684,10 @@ g_key_file_parse_line (GKeyFile     *key_file,
 				     &parse_error);
   else
     {
-      gchar *line_utf8 = _g_utf8_make_valid (line);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_PARSE,
                    _("Key file contains line '%s' which is not "
-                     "a key-value pair, group, or comment"), 
-		   line_utf8);
-      g_free (line_utf8);
-
+                     "a key-value pair, group, or comment"), line);
       return;
     }
 
@@ -805,12 +795,9 @@ g_key_file_parse_key_value_pair (GKeyFile     *key_file,
     {
       if (g_ascii_strcasecmp (value, "UTF-8") != 0)
         {
-	  gchar *value_utf8 = _g_utf8_make_valid (value);
           g_set_error (error, G_KEY_FILE_ERROR,
                        G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
-                       _("Key file contains unsupported "
-			 "encoding '%s'"), value_utf8);
-	  g_free (value_utf8);
+                       _("Key file contains unsupported encoding '%s'"), value);
 
           g_free (key);
           g_free (value);
@@ -936,6 +923,7 @@ g_key_file_to_data (GKeyFile  *key_file,
 		    GError   **error)
 {
   GString *data_string;
+  gchar *data;
   GList *group_node, *key_file_node;
 
   g_return_val_if_fail (key_file != NULL, NULL);
@@ -952,9 +940,6 @@ g_key_file_to_data (GKeyFile  *key_file,
 
       if (group->comment != NULL)
         g_string_append_printf (data_string, "%s\n", group->comment->value);
-      else if (group_node->next) /* separate groups by at least an empty line */
-        g_string_append_c (data_string, '\n');
-
       if (group->name != NULL)
         g_string_append_printf (data_string, "[%s]\n", group->name);
 
@@ -976,7 +961,11 @@ g_key_file_to_data (GKeyFile  *key_file,
   if (length)
     *length = data_string->len;
 
-  return g_string_free (data_string, FALSE);
+  data = data_string->str;
+
+  g_string_free (data_string, FALSE);
+
+  return data;
 }
 
 /**
@@ -1022,33 +1011,20 @@ g_key_file_get_keys (GKeyFile     *key_file,
       return NULL;
     }
 
-  num_keys = 0;
-  for (tmp = group->key_value_pairs; tmp; tmp = tmp->next)
-    {
-      GKeyFileKeyValuePair *pair;
-
-      pair = (GKeyFileKeyValuePair *) tmp->data;
-
-      if (pair->key)
-	num_keys++;
-    }
+  num_keys = g_list_length (group->key_value_pairs);
   
-  keys = g_new0 (gchar *, num_keys + 1);
+  keys = (gchar **) g_new0 (gchar **, num_keys + 1);
 
-  i = num_keys - 1;
-  for (tmp = group->key_value_pairs; tmp; tmp = tmp->next)
+  tmp = group->key_value_pairs;
+  for (i = 1; i <= num_keys; i++)
     {
       GKeyFileKeyValuePair *pair;
 
       pair = (GKeyFileKeyValuePair *) tmp->data;
+      keys[num_keys - i] = g_strdup (pair->key);
 
-      if (pair->key)
-	{
-	  keys[i] = g_strdup (pair->key);
-	  i--;
-	}
+      tmp = tmp->next;
     }
-
   keys[num_keys] = NULL;
 
   if (length)
@@ -1110,7 +1086,7 @@ g_key_file_get_groups (GKeyFile *key_file,
    * list) is always the comment group at the top,
    * which we skip
    */
-  groups = g_new0 (gchar *, num_groups);
+  groups = (gchar **) g_new0 (gchar **, num_groups);
 
   group_node = g_list_last (key_file->groups);
   
@@ -1152,7 +1128,7 @@ g_key_file_get_groups (GKeyFile *key_file,
  * and @error is set to #G_KEY_FILE_ERROR_GROUP_NOT_FOUND.
  *
  * Return value: a newly allocated string or %NULL if the specified 
- *  key cannot be found.
+ * key cannot be found.
  *
  * Since: 2.6
  **/
@@ -1258,7 +1234,7 @@ g_key_file_set_value (GKeyFile    *key_file,
  * and @error is set to #G_KEY_FILE_ERROR_GROUP_NOT_FOUND.
  *
  * Return value: a newly allocated string or %NULL if the specified 
- *   key cannot be found.
+ * key cannot be found.
  *
  * Since: 2.6
  **/
@@ -1287,14 +1263,11 @@ g_key_file_get_string (GKeyFile     *key_file,
 
   if (!g_utf8_validate (value, -1, NULL))
     {
-      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
                    _("Key file contains key '%s' with value '%s' "
-                     "which is not UTF-8"), key, value_utf8);
-      g_free (value_utf8);
+                     "which is not UTF-8"), key, value);
       g_free (value);
-
       return NULL;
     }
 
@@ -1399,14 +1372,11 @@ g_key_file_get_string_list (GKeyFile     *key_file,
 
   if (!g_utf8_validate (value, -1, NULL))
     {
-      gchar *value_utf8 = _g_utf8_make_valid (value);
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_UNKNOWN_ENCODING,
                    _("Key file contains key '%s' with value '%s' "
-                     "which is not UTF-8"), key, value_utf8);
-      g_free (value_utf8);
+                     "which is not UTF-8"), key, value);
       g_free (value);
-
       return NULL;
     }
 
@@ -1475,7 +1445,7 @@ g_key_file_set_string_list (GKeyFile            *key_file,
   g_return_if_fail (list != NULL);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; i < length && list[i] != NULL; i++)
+  for (i = 0; list[i] != NULL && i < length; i++)
     {
       gchar *value;
 
@@ -1545,9 +1515,9 @@ extern GSList *_g_compute_locale_variants (const gchar *locale);
  * with @key cannot be interpreted or no suitable translation can
  * be found then the untranslated value is returned.
  *
- * Return value: a newly allocated string or %NULL if the specified 
- *   key cannot be found.
- *
+ * Return value: a newly allocated string or %NULL if the specified key 
+ *  cannot be found.
+ * 
  * Since: 2.6
  **/
 gchar *
@@ -1600,7 +1570,7 @@ g_key_file_get_locale_string (GKeyFile     *key_file,
 						candidate_key, NULL);
       g_free (candidate_key);
 
-      if (translated_value)
+      if (translated_value && g_utf8_validate (translated_value, -1, NULL))
 	break;
 
       g_free (translated_value);
@@ -1635,7 +1605,8 @@ g_key_file_get_locale_string (GKeyFile     *key_file,
  *
  * Returns the values associated with @key under @group_name
  * translated in the given @locale if available.  If @locale is
- * %NULL then the current locale is assumed.
+ * %NULL then the current locale is assumed. If @group_name is
+ * %NULL, then the start group is used.
 
  * If @key cannot be found then %NULL is returned and @error is set to
  * #G_KEY_FILE_ERROR_KEY_NOT_FOUND. If the values associated
@@ -1700,7 +1671,8 @@ g_key_file_get_locale_string_list (GKeyFile     *key_file,
  *
  * Associates a list of string values for @key and @locale under
  * @group_name.  If the translation for @key cannot be found then
- * it is created. 
+ * it is created. If @group_name is %NULL, the start group is
+ * used.
  *
  * Since: 2.6
  **/
@@ -1723,7 +1695,7 @@ g_key_file_set_locale_string_list (GKeyFile            *key_file,
   g_return_if_fail (length != 0);
 
   value_list = g_string_sized_new (length * 128);
-  for (i = 0; i < length && list[i] != NULL; i++)
+  for (i = 0; list[i] != NULL && i < length; i++)
     {
       gchar *value;
       
@@ -1749,7 +1721,7 @@ g_key_file_set_locale_string_list (GKeyFile            *key_file,
  * @error: return location for a #GError
  *
  * Returns the value associated with @key under @group_name as a
- * boolean. 
+ * boolean. If @group_name is %NULL, the start group is used.
  *
  * If @key cannot be found then the return value is undefined and
  * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
@@ -1814,7 +1786,8 @@ g_key_file_get_boolean (GKeyFile     *key_file,
  * @value: %TRUE or %FALSE
  *
  * Associates a new boolean value with @key under @group_name.
- * If @key cannot be found then it is created. 
+ * If @key cannot be found then it is created. If @group_name
+ * is %NULL, the start group is used.
  *
  * Since: 2.6
  **/
@@ -2030,7 +2003,8 @@ g_key_file_get_integer (GKeyFile     *key_file,
  * @value: an integer value
  *
  * Associates a new integer value with @key under @group_name.
- * If @key cannot be found then it is created.
+ * If @key cannot be found then it is created. If @group_name
+ * is %NULL, the start group is used.
  *
  * Since: 2.6
  **/
@@ -2060,7 +2034,7 @@ g_key_file_set_integer (GKeyFile    *key_file,
  * @error: return location for a #GError
  *
  * Returns the values associated with @key under @group_name as
- * integers. 
+ * integers. If @group_name is %NULL, the start group is used.
  *
  * If @key cannot be found then the return value is undefined and
  * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
@@ -2132,6 +2106,7 @@ g_key_file_get_integer_list (GKeyFile     *key_file,
  *
  * Associates a list of integer values with @key under
  * @group_name.  If @key cannot be found then it is created.
+ * If @group_name is %NULL the start group is used.
  *
  * Since: 2.6
  **/
@@ -2167,218 +2142,6 @@ g_key_file_set_integer_list (GKeyFile     *key_file,
   g_string_free (values, TRUE);
 }
 
-/**
- * g_key_file_get_double:
- * @key_file: a #GKeyFile
- * @group_name: a group name
- * @key: a key
- * @error: return location for a #GError
- *
- * Returns the value associated with @key under @group_name as an
- * integer. If @group_name is %NULL, the start_group is used.
- *
- * If @key cannot be found then the return value is undefined and
- * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
- * the value associated with @key cannot be interpreted as a double
- * then the return value is also undefined and @error is set to
- * #G_KEY_FILE_ERROR_INVALID_VALUE.
- *
- * Return value: the value associated with the key as a double.
- *
- * Since: 2.12
- **/
-gdouble
-g_key_file_get_double  (GKeyFile     *key_file,
-                        const gchar  *group_name,
-                        const gchar  *key,
-                        GError      **error)
-{
-  GError *key_file_error;
-  gchar *value;
-  gdouble double_value;
-
-  g_return_val_if_fail (key_file != NULL, -1);
-  g_return_val_if_fail (group_name != NULL, -1);
-  g_return_val_if_fail (key != NULL, -1);
-
-  key_file_error = NULL;
-
-  value = g_key_file_get_value (key_file, group_name, key, &key_file_error);
-
-  if (key_file_error)
-    {
-      g_propagate_error (error, key_file_error);
-      return 0;
-    }
-
-  double_value = g_key_file_parse_value_as_double (key_file, value,
-                                                  &key_file_error);
-  g_free (value);
-
-  if (key_file_error)
-    {
-      if (g_error_matches (key_file_error,
-                           G_KEY_FILE_ERROR,
-                           G_KEY_FILE_ERROR_INVALID_VALUE))
-        {
-          g_set_error (error, G_KEY_FILE_ERROR,
-                       G_KEY_FILE_ERROR_INVALID_VALUE,
-                       _("Key file contains key '%s' in group '%s' "
-                         "which has value that cannot be interpreted."), key,
-                       group_name);
-          g_error_free (key_file_error);
-        }
-      else
-        g_propagate_error (error, key_file_error);
-    }
-
-  return double_value;
-}
-
-/**
- * g_key_file_set_double:
- * @key_file: a #GKeyFile
- * @group_name: a group name
- * @key: a key
- * @value: an double value
- *
- * Associates a new double value with @key under @group_name.
- * If @key cannot be found then it is created. If @group_name
- * is %NULL, the start group is used.
- *
- * Since: 2.12
- **/
-void
-g_key_file_set_double  (GKeyFile    *key_file,
-                        const gchar *group_name,
-                        const gchar *key,
-                        gdouble      value)
-{
-  gchar result[G_ASCII_DTOSTR_BUF_SIZE];
-
-  g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
-
-  g_ascii_dtostr ( result, sizeof (result), value );
-  g_key_file_set_value (key_file, group_name, key, result);
-}
-
-/**
- * g_key_file_get_double_list:
- * @key_file: a #GKeyFile
- * @group_name: a group name
- * @key: a key
- * @length: the number of doubles returned
- * @error: return location for a #GError
- *
- * Returns the values associated with @key under @group_name as
- * doubles. If @group_name is %NULL, the start group is used.
- *
- * If @key cannot be found then the return value is undefined and
- * @error is set to #G_KEY_FILE_ERROR_KEY_NOT_FOUND. Likewise, if
- * the values associated with @key cannot be interpreted as doubles
- * then the return value is also undefined and @error is set to
- * #G_KEY_FILE_ERROR_INVALID_VALUE.
- *
- * Return value: the values associated with the key as a double
- *
- * Since: 2.12
- **/
-gdouble *
-g_key_file_get_double_list  (GKeyFile     *key_file,
-                             const gchar  *group_name,
-                             const gchar  *key,
-                             gsize        *length,
-                             GError      **error)
-{
-  GError *key_file_error = NULL;
-  gchar **values;
-  gdouble *double_values;
-  gsize i, num_doubles;
-
-  g_return_val_if_fail (key_file != NULL, NULL);
-  g_return_val_if_fail (group_name != NULL, NULL);
-  g_return_val_if_fail (key != NULL, NULL);
-
-  values = g_key_file_get_string_list (key_file, group_name, key,
-                                       &num_doubles, &key_file_error);
-
-  if (key_file_error)
-    g_propagate_error (error, key_file_error);
-
-  if (!values)
-    return NULL;
-
-  double_values = g_new0 (gdouble, num_doubles);
-
-  for (i = 0; i < num_doubles; i++)
-    {
-      double_values[i] = g_key_file_parse_value_as_double (key_file,
-							   values[i],
-							   &key_file_error);
-
-      if (key_file_error)
-        {
-          g_propagate_error (error, key_file_error);
-          g_strfreev (values);
-          g_free (double_values);
-
-          return NULL;
-        }
-    }
-  g_strfreev (values);
-
-  if (length)
-    *length = num_doubles;
-
-  return double_values;
-}
-
-/**
- * g_key_file_set_double_list:
- * @key_file: a #GKeyFile
- * @group_name: a group name
- * @key: a key
- * @list: an array of double values
- * @length: number of double values in @list
- *
- * Associates a list of double values with @key under
- * @group_name.  If @key cannot be found then it is created.
- * If @group_name is %NULL the start group is used.
- *
- * Since: 2.12
- **/
-void
-g_key_file_set_double_list (GKeyFile     *key_file,
-			    const gchar  *group_name,
-			    const gchar  *key,
-			    gdouble       list[],
-			    gsize         length)
-{
-  GString *values;
-  gsize i;
-
-  g_return_if_fail (key_file != NULL);
-  g_return_if_fail (group_name != NULL);
-  g_return_if_fail (key != NULL);
-  g_return_if_fail (list != NULL);
-
-  values = g_string_sized_new (length * 16);
-  for (i = 0; i < length; i++)
-    {
-      gchar result[G_ASCII_DTOSTR_BUF_SIZE];
-
-      g_ascii_dtostr( result, sizeof (result), list[i] );
-
-      g_string_append (values, result);
-      g_string_append_c (values, ';');
-    }
-
-  g_key_file_set_value (key_file, group_name, key, values->str);
-  g_string_free (values, TRUE);
-}
-
 static void
 g_key_file_set_key_comment (GKeyFile             *key_file,
                             const gchar          *group_name,
@@ -2569,7 +2332,6 @@ g_key_file_get_key_comment (GKeyFile             *key_file,
                             GError              **error)
 {
   GKeyFileGroup *group;
-  GKeyFileKeyValuePair *pair;
   GList *key_node, *tmp;
   GString *string;
   gchar *comment;
@@ -2605,37 +2367,23 @@ g_key_file_get_key_comment (GKeyFile             *key_file,
    * key and concatentate them.
    */
   tmp = key_node->next;
-  if (!key_node->next)
-    return NULL;
-
-  pair = (GKeyFileKeyValuePair *) tmp->data;
-  if (pair->key != NULL)
-    return NULL;
-
-  while (tmp->next)
-    {
-      pair = (GKeyFileKeyValuePair *) tmp->next->data;
-      
-      if (pair->key != NULL)
-        break;
-
-      tmp = tmp->next;
-    }
-
-  while (tmp != key_node)
+  while (tmp != NULL)
     {
       GKeyFileKeyValuePair *pair;
-      
+
       pair = (GKeyFileKeyValuePair *) tmp->data;
+
+      if (pair->key != NULL)
+        break;
       
       if (string == NULL)
-	string = g_string_sized_new (512);
-      
+        string = g_string_sized_new (512);
+
       comment = g_key_file_parse_value_as_comment (key_file, pair->value);
       g_string_append (string, comment);
       g_free (comment);
-      
-      tmp = tmp->prev;
+
+      tmp = tmp->next;
     }
 
   if (string != NULL)
@@ -2649,68 +2397,15 @@ g_key_file_get_key_comment (GKeyFile             *key_file,
   return comment;
 }
 
-static gchar *
-get_group_comment (GKeyFile       *key_file,
-		   GKeyFileGroup  *group,
-		   GError        **error)
-{
-  GString *string;
-  GList *tmp;
-  gchar *comment;
-
-  string = NULL;
-
-  tmp = group->key_value_pairs;
-  while (tmp)
-    {
-      GKeyFileKeyValuePair *pair;
-
-      pair = (GKeyFileKeyValuePair *) tmp->data;
-
-      if (pair->key != NULL)
-	{
-	  tmp = tmp->prev;
-	  break;
-	}
-
-      if (tmp->next == NULL)
-	break;
-
-      tmp = tmp->next;
-    }
-  
-  while (tmp != NULL)
-    {
-      GKeyFileKeyValuePair *pair;
-
-      pair = (GKeyFileKeyValuePair *) tmp->data;
-
-      if (string == NULL)
-        string = g_string_sized_new (512);
-
-      comment = g_key_file_parse_value_as_comment (key_file, pair->value);
-      g_string_append (string, comment);
-      g_free (comment);
-
-      tmp = tmp->prev;
-    }
-
-  if (string != NULL)
-    return g_string_free (string, FALSE);
-
-  return NULL;
-}
-
 static gchar *
 g_key_file_get_group_comment (GKeyFile             *key_file,
                               const gchar          *group_name,
                               GError              **error)
 {
-  GList *group_node;
   GKeyFileGroup *group;
   
-  group_node = g_key_file_lookup_group_node (key_file, group_name);
-  if (!group_node)
+  group = g_key_file_lookup_group (key_file, group_name);
+  if (!group)
     {
       g_set_error (error, G_KEY_FILE_ERROR,
                    G_KEY_FILE_ERROR_GROUP_NOT_FOUND,
@@ -2720,21 +2415,20 @@ g_key_file_get_group_comment (GKeyFile             *key_file,
       return NULL;
     }
 
-  group = (GKeyFileGroup *)group_node->data;
   if (group->comment)
     return g_strdup (group->comment->value);
-  
-  group_node = group_node->next;
-  group = (GKeyFileGroup *)group_node->data;  
-  return get_group_comment (key_file, group, error);
+
+  return NULL;
 }
 
 static gchar *
 g_key_file_get_top_comment (GKeyFile             *key_file,
                             GError              **error)
 {
-  GList *group_node;
+  GList *group_node, *tmp;
   GKeyFileGroup *group;
+  GString *string;
+  gchar *comment;
 
   /* The last group in the list should be the top (comments only)
    * group in the file
@@ -2744,7 +2438,40 @@ g_key_file_get_top_comment (GKeyFile             *key_file,
   group = (GKeyFileGroup *) group_node->data;
   g_assert (group->name == NULL);
 
-  return get_group_comment (key_file, group, error);
+  string = NULL;
+
+  /* Then find all the comments already associated with the
+   * key and concatentate them.
+   */
+  tmp = group->key_value_pairs;
+  while (tmp != NULL)
+    {
+      GKeyFileKeyValuePair *pair;
+
+      pair = (GKeyFileKeyValuePair *) tmp->data;
+
+      if (pair->key != NULL)
+        break;
+      
+      if (string == NULL)
+        string = g_string_sized_new (512);
+
+      comment = g_key_file_parse_value_as_comment (key_file, pair->value);
+      g_string_append (string, comment);
+      g_free (comment);
+
+      tmp = tmp->next;
+    }
+
+  if (string != NULL)
+    {
+      comment = string->str;
+      g_string_free (string, FALSE);
+    }
+  else
+    comment = NULL;
+
+  return comment;
 }
 
 /**
@@ -2754,15 +2481,14 @@ g_key_file_get_top_comment (GKeyFile             *key_file,
  * @key: a key
  * @error: return location for a #GError
  *
- * Retrieves a comment above @key from @group_name.
+ * Retreives a comment above @key from @group_name.
  * @group_name. If @key is %NULL then @comment will
  * be read from above @group_name.  If both @key
  * and @group_name are NULL, then @comment will
  * be read from above the first group in the file.
  *
- * Returns: a comment that should be freed with g_free()
- *
  * Since: 2.6
+ * Returns: a comment that should be freed with g_free()
  **/
 gchar * 
 g_key_file_get_comment (GKeyFile             *key_file,
@@ -2841,7 +2567,8 @@ g_key_file_has_group (GKeyFile    *key_file,
  * @error: return location for a #GError
  *
  * Looks whether the key file has the key @key in the group
- * @group_name. 
+ * @group_name. If @group_name is %NULL, the start group is
+ * used.
  *
  * Return value: %TRUE if @key is a part of @group_name, %FALSE
  * otherwise.
@@ -2886,9 +2613,7 @@ g_key_file_add_group (GKeyFile    *key_file,
 
   g_return_if_fail (key_file != NULL);
   g_return_if_fail (group_name != NULL);
-
-  if (g_key_file_lookup_group_node (key_file, group_name) != NULL)
-    return;
+  g_return_if_fail (g_key_file_lookup_group_node (key_file, group_name) == NULL);
 
   group = g_new0 (GKeyFileGroup, 1);
   group->name = g_strdup (group_name);
@@ -3130,12 +2855,15 @@ g_key_file_lookup_group_node (GKeyFile    *key_file,
   GKeyFileGroup *group;
   GList *tmp;
 
+  group = NULL;
   for (tmp = key_file->groups; tmp != NULL; tmp = tmp->next)
     {
       group = (GKeyFileGroup *) tmp->data;
 
       if (group && group->name && strcmp (group->name, group_name) == 0)
         break;
+
+      group = NULL;
     }
 
   return tmp;
@@ -3209,9 +2937,14 @@ g_key_file_line_is_group (const gchar *line)
 
   p = g_utf8_next_char (p);
 
+  if (!*p)
+    return FALSE;
+
+  p = g_utf8_next_char (p);
+
   /* Group name must be non-empty
    */
-  if (!*p || *p == ']')
+  if (*p == ']')
     return FALSE;
 
   while (*p && *p != ']')
@@ -3281,13 +3014,6 @@ g_key_file_parse_value_as_string (GKeyFile     *key_file,
               *q = '\\';
               break;
 
-	    case '\0':
-	      g_set_error (error, G_KEY_FILE_ERROR,
-			   G_KEY_FILE_ERROR_INVALID_VALUE,
-			   _("Key file contains escape character "
-			     "at end of line"));
-	      break;
-
             default:
 	      if (pieces && *p == key_file->list_separator)
 		*q = key_file->list_separator;
@@ -3323,13 +3049,15 @@ g_key_file_parse_value_as_string (GKeyFile     *key_file,
 	    }
 	}
 
-      if (*p == '\0')
-	break;
-
       q++;
       p++;
     }
 
+  if (p > value && p[-1] == '\\' && q[-1] != '\\' && *error == NULL)
+    g_set_error (error, G_KEY_FILE_ERROR,
+		 G_KEY_FILE_ERROR_INVALID_VALUE,
+		 _("Key file contains escape character at end of line"));
+
   *q = '\0';
   if (pieces)
   {
@@ -3437,38 +3165,15 @@ g_key_file_parse_value_as_integer (GKeyFile     *key_file,
 				   GError      **error)
 {
   gchar *end_of_valid_int;
-  glong long_value;
-  gint int_value;
+  gint int_value = 0;
 
-  errno = 0;
-  long_value = strtol (value, &end_of_valid_int, 10);
+  int_value = strtol (value, &end_of_valid_int, 10);
 
-  if (*value == '\0' || *end_of_valid_int != '\0')
-    {
-      gchar *value_utf8 = _g_utf8_make_valid (value);
-      g_set_error (error, G_KEY_FILE_ERROR,
-		   G_KEY_FILE_ERROR_INVALID_VALUE,
-		   _("Value '%s' cannot be interpreted "
-		     "as a number."), value_utf8);
-      g_free (value_utf8);
+  if (*end_of_valid_int != '\0')
+    g_set_error (error, G_KEY_FILE_ERROR,
+		 G_KEY_FILE_ERROR_INVALID_VALUE,
+		 _("Value '%s' cannot be interpreted as a number."), value);
 
-      return 0;
-    }
-
-  int_value = long_value;
-  if (int_value != long_value || errno == ERANGE)
-    {
-      gchar *value_utf8 = _g_utf8_make_valid (value);
-      g_set_error (error,
-		   G_KEY_FILE_ERROR, 
-		   G_KEY_FILE_ERROR_INVALID_VALUE,
-		   _("Integer value '%s' out of range"), 
-		   value_utf8);
-      g_free (value_utf8);
-
-      return 0;
-    }
-  
   return int_value;
 }
 
@@ -3480,37 +3185,11 @@ g_key_file_parse_integer_as_value (GKeyFile *key_file,
   return g_strdup_printf ("%d", value);
 }
 
-static gdouble
-g_key_file_parse_value_as_double  (GKeyFile     *key_file,
-                                   const gchar  *value,
-                                   GError      **error)
-{
-  gchar *end_of_valid_d;
-  gdouble double_value = 0;
-
-  double_value = g_ascii_strtod (value, &end_of_valid_d);
-
-  if (*end_of_valid_d != '\0' || end_of_valid_d == value)
-    {
-      gchar *value_utf8 = _g_utf8_make_valid (value);
-      g_set_error (error, G_KEY_FILE_ERROR,
-		   G_KEY_FILE_ERROR_INVALID_VALUE,
-		   _("Value '%s' cannot be interpreted "
-		     "as a float number."), 
-		   value_utf8);
-      g_free (value_utf8);
-    }
-
-  return double_value;
-}
-
 static gboolean
 g_key_file_parse_value_as_boolean (GKeyFile     *key_file,
 				   const gchar  *value,
 				   GError      **error)
 {
-  gchar *value_utf8;
-
   if (value)
     {
       if (strcmp (value, "true") == 0 || strcmp (value, "1") == 0)
@@ -3519,12 +3198,9 @@ g_key_file_parse_value_as_boolean (GKeyFile     *key_file,
         return FALSE;
     }
 
-  value_utf8 = _g_utf8_make_valid (value);
   g_set_error (error, G_KEY_FILE_ERROR,
                G_KEY_FILE_ERROR_INVALID_VALUE,
-               _("Value '%s' cannot be interpreted "
-		 "as a boolean."), value_utf8);
-  g_free (value_utf8);
+               _("Value '%s' cannot be interpreted as a boolean."), value);
 
   return FALSE;
 }
@@ -3544,7 +3220,7 @@ g_key_file_parse_value_as_comment (GKeyFile    *key_file,
                                    const gchar *value)
 {
   GString *string;
-  gchar **lines;
+  gchar **lines, *comment;
   gsize i;
 
   string = g_string_sized_new (512);
@@ -3560,7 +3236,11 @@ g_key_file_parse_value_as_comment (GKeyFile    *key_file,
     }
   g_strfreev (lines);
 
-  return g_string_free (string, FALSE);
+  comment = string->str;
+
+  g_string_free (string, FALSE);
+
+  return comment;
 }
 
 static gchar *
@@ -3568,7 +3248,7 @@ g_key_file_parse_comment_as_value (GKeyFile      *key_file,
                                    const gchar   *comment)
 {
   GString *string;
-  gchar **lines;
+  gchar **lines, *value;
   gsize i;
 
   string = g_string_sized_new (512);
@@ -3580,8 +3260,11 @@ g_key_file_parse_comment_as_value (GKeyFile      *key_file,
                             lines[i + 1] == NULL? "" : "\n");
   g_strfreev (lines);
 
-  return g_string_free (string, FALSE);
+  value = string->str;
+
+  g_string_free (string, FALSE);
+
+  return value;
 }
 
-#define __G_KEY_FILE_C__
-#include "galiasdef.c"
+
